importFrom("graphics", "text", "plot", "abline", "title", "par", "arrows", "lines", "locator", "points")
importFrom("stats", "D", "as.formula", "coef", "formula", "logLik", "qnorm", "time", "nobs", "vcov", "confint", "deviance", "is.ts", "na.omit", "printCoefmat", "pt", "pchisq", "qchisq", "qt", "quantile", "setNames", "predict")
importFrom("utils", "str", ".DollarNames", "packageVersion")
importFrom("Rdpack", reprompt)
importFrom("MASS", "ginv")
importFrom("Matrix", "nearPD")
importFrom(numDeriv, jacobian)
importFrom(xtable, xtable)
importFrom(latex2exp, TeX)
importFrom(grid, grid.newpage, pushViewport, viewport, grid.layout)
importFrom(reshape2, melt)
importFrom(plyr, ddply, is.formula)
#importFrom(dplyr, group_by, mutate_all)
importFrom(magrittr, '%>%')
importFrom(mice, mice, complete)
importFrom("stringi", "stri_startswith_fixed", "stri_endswith_fixed")
import(ggplot2)
import(methods)
import(fda)
import(car)
import(deSolve)
exportPattern("^dynr.+")
exportPattern("^prep.+")
#exportPattern("^[^\\.]") #export everything
#exportPattern("^[[:alpha:]]+") #export everything that doesn't start with "."
#exportClassPattern("^dynr[[A-Z]][[:alpha:]]+") #export all dynr* Classes
export(plotFormula)
export(printex)
export(diag)
export(plotGCV)
export(getdx)
export(theta_plot)
exportMethods("print", "show")
S3method(logLik, dynrCook)
S3method(nobs, dynrCook)
S3method(nobs, dynrModel)
S3method(coef, dynrCook)
S3method(coef, dynrModel)
S3method(vcov, dynrCook)
S3method(predict, dynrModel)
S3method(confint, dynrCook)
S3method(.DollarNames, dynrModel)
S3method(.DollarNames, dynrCook)
S3method(summary, dynrCook)
S3method(print, summary.dynrCook)
S3method(coef, summary.dynrCook)
S3method(autoplot, dynrCook)
S3method(autoplot, dynrTaste)
S3method(plot, dynrCook)
S3method(deviance, dynrCook)
export(`coef<-`)
S3method(`coef<-`, dynrModel)
useDynLib(dynr, .registration=TRUE)

